import pygame
import random

# 색상 정의
BACK = (153, 0, 153)
WHITE = (255, 255, 255)
COLORS = [
    (0, 255, 255),  # 시안 (I)
    (255, 255, 0),  # 노랑 (O)
    (230, 0, 230),  # 보라 (T)
    (0, 255, 0),    # 초록 (S)
    (255, 0, 0),    # 빨강 (Z)
    (0, 100, 255),  # 파랑 (J)
    (255, 127, 0)   # 주황 (L)
]

# 게임 설정
BLOCK_SIZE = 30
BOARD_WIDTH = 10
BOARD_HEIGHT = 20
SCREEN_WIDTH = BOARD_WIDTH * BLOCK_SIZE + 200
SCREEN_HEIGHT = BOARD_HEIGHT * BLOCK_SIZE

# 테트로미노 모양
SHAPES = [
    [[1, 1, 1, 1]],
    [[1, 1], [1, 1]],
    [[1, 1, 1], [0, 1, 0]],
    [[1, 1, 0], [0, 1, 1]],
    [[0, 1, 1], [1, 1, 0]],
    [[1, 1, 1], [1, 0, 0]],
    [[1, 1, 1], [0, 0, 1]]
]

class Tetromino:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.shape = random.choice(SHAPES)
        self.color = random.choice(COLORS)
        self.rotation = 0

    def rotate(self):
        self.rotation = (self.rotation + 1) % len(self.shape)
        self.shape = list(zip(*self.shape[::-1]))

class Tetris:
    def __init__(self):
        self.board = [[0 for _ in range(BOARD_WIDTH)] for _ in range(BOARD_HEIGHT)]
        self.current_piece = None
        self.next_piece = None
        self.game_over = False
        self.score = 0
        self.level = 1
        self.new_piece()

    def new_piece(self):
        if not self.next_piece:
            self.next_piece = Tetromino(BOARD_WIDTH // 2 - 1, 0)
        self.current_piece = self.next_piece
        self.next_piece = Tetromino(BOARD_WIDTH // 2 - 1, 0)
        if self.collision():
            self.game_over = True

    def collision(self):
        for y, row in enumerate(self.current_piece.shape):
            for x, cell in enumerate(row):
                if cell:
                    if (self.current_piece.x + x < 0 or
                        self.current_piece.x + x >= BOARD_WIDTH or
                        self.current_piece.y + y >= BOARD_HEIGHT or
                        self.board[self.current_piece.y + y][self.current_piece.x + x]):
                        return True
        return False

    def move(self, dx, dy):
        self.current_piece.x += dx
        self.current_piece.y += dy
        if self.collision():
            self.current_piece.x -= dx
            self.current_piece.y -= dy
            return False
        return True

    def rotate_piece(self):
        old_shape = self.current_piece.shape
        self.current_piece.rotate()
        if self.collision():
            self.current_piece.shape = old_shape

    def drop_piece(self):
        while self.move(0, 1):
            pass
        self.place_piece()

    def place_piece(self):
        for y, row in enumerate(self.current_piece.shape):
            for x, cell in enumerate(row):
                if cell:
                    self.board[self.current_piece.y + y][self.current_piece.x + x] = self.current_piece.color
        self.clear_lines()
        self.new_piece()

    def clear_lines(self):
        lines_cleared = 0
        for i, row in enumerate(self.board):
            if all(row):
                del self.board[i]
                self.board.insert(0, [0 for _ in range(BOARD_WIDTH)])
                lines_cleared += 1
        self.score += lines_cleared ** 2 * 100
        self.level = self.score // 1000 + 1

    def reset(self):
        self.__init__()

def draw_block(surface, x, y, color):
    r, g, b = color
    gradient_top = (min(r + 50, 255), min(g + 50, 255), min(b + 50, 255))
    gradient_bottom = (max(r - 30, 0), max(g - 30, 0), max(b - 30, 0))
    
    for i in range(BLOCK_SIZE):
        current_color = [
            int(gradient_top[j] + (gradient_bottom[j] - gradient_top[j]) * i / BLOCK_SIZE)
            for j in range(3)
        ]
        pygame.draw.rect(surface, current_color, (x, y + i, BLOCK_SIZE, 1))
    
    # 블록 테두리에 밝은 효과 추가
    pygame.draw.rect(surface, gradient_top, (x, y, BLOCK_SIZE, BLOCK_SIZE), 1)
    # 모서리 둥글게 처리
    pygame.draw.circle(surface, gradient_top, (x + 2, y + 2), 2)
    pygame.draw.circle(surface, gradient_top, (x + BLOCK_SIZE - 2, y + 2), 2)
    pygame.draw.circle(surface, gradient_bottom, (x + 2, y + BLOCK_SIZE - 2), 2)
    pygame.draw.circle(surface, gradient_bottom, (x + BLOCK_SIZE - 2, y + BLOCK_SIZE - 2), 2)

def draw_board(surface, board):
    for y, row in enumerate(board):
        for x, color in enumerate(row):
            if color:
                draw_block(surface, x * BLOCK_SIZE, y * BLOCK_SIZE, color)

def draw_current_piece(surface, piece):
    for i, row in enumerate(piece.shape):
        for j, cell in enumerate(row):
            if cell:
                draw_block(surface, (piece.x + j) * BLOCK_SIZE, (piece.y + i) * BLOCK_SIZE, piece.color)

def draw_next_piece(surface, piece):
    for i, row in enumerate(piece.shape):
        for j, cell in enumerate(row):
            if cell:
                draw_block(surface, SCREEN_WIDTH - 100 + j * 20, 50 + i * 20, piece.color)

def draw_text(surface, text, size, x, y):
    font = pygame.font.Font(None, size)
    text_surface = font.render(text, True, WHITE)
    text_rect = text_surface.get_rect()
    text_rect.midtop = (x, y)
    surface.blit(text_surface, text_rect)

def show_intro_screen(screen):
    screen.fill(BACK)
    draw_text(screen, "TETRIS GAME", 64, SCREEN_WIDTH // 2, SCREEN_HEIGHT // 2 - 48)
    draw_text(screen, "Press any key to start", 36, SCREEN_WIDTH // 2, SCREEN_HEIGHT // 2 + 24)
    pygame.display.flip()
    waiting = True
    while waiting:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                return False
            if event.type == pygame.KEYUP:
                waiting = False
    return True

def main():
    pygame.init()
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
    pygame.display.set_caption('테트리스')

    clock = pygame.time.Clock()
    game = Tetris()

    fall_time = 0
    fall_speed = 0.5
    running = True

    # 배경 그라데이션 생성
    background = pygame.Surface((SCREEN_WIDTH, SCREEN_HEIGHT))
    for y in range(SCREEN_HEIGHT):
        r = int(70 + (0 - 70) * y / SCREEN_HEIGHT)
        g = int(0 + (0 - 0) * y / SCREEN_HEIGHT)
        b = int(130 + (50 - 130) * y / SCREEN_HEIGHT)
        pygame.draw.line(background, (r, g, b), (0, y), (SCREEN_WIDTH, y))

    # 반투명한 정보창 배경 생성
    info_panel = pygame.Surface((200, SCREEN_HEIGHT))
    info_panel.set_alpha(128)  # 0-255 사이의 값, 128은 반투명
    info_panel.fill((30, 30, 30))  # 약간 어두운 회색

    if not show_intro_screen(screen):
        running = False

    while running:
        if not game.game_over:
            fall_time += clock.get_rawtime()
            clock.tick()

            if fall_time / 1000 > fall_speed:
                if not game.move(0, 1):
                    game.place_piece()
                fall_time = 0

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            if event.type == pygame.KEYDOWN:
                if not game.game_over:
                    if event.key == pygame.K_LEFT:
                        game.move(-1, 0)
                    if event.key == pygame.K_RIGHT:
                        game.move(1, 0)
                    if event.key == pygame.K_DOWN:
                        game.drop_piece()
                    if event.key == pygame.K_UP:
                        game.rotate_piece()
                if event.key == pygame.K_r:
                    game.reset()

        screen.blit(background, (0, 0))
        screen.blit(info_panel, (BOARD_WIDTH * BLOCK_SIZE, 0))
        
        draw_board(screen, game.board)
        if not game.game_over:
            draw_current_piece(screen, game.current_piece)
        
        draw_next_piece(screen, game.next_piece)

        score_text = f'Score: {game.score}'
        level_text = f'Level: {game.level}'
        next_text = 'Next:'
        draw_text(screen, score_text, 36, SCREEN_WIDTH - 100, 200)
        draw_text(screen, level_text, 36, SCREEN_WIDTH - 100, 240)
        draw_text(screen, next_text, 36, SCREEN_WIDTH - 100, 10)

        if game.game_over:
            draw_text(screen, "GAME OVER", 64, SCREEN_WIDTH // 2, SCREEN_HEIGHT // 2 - 48)
            draw_text(screen, "Press R to Restart", 36, SCREEN_WIDTH // 2, SCREEN_HEIGHT // 2 + 24)

        pygame.display.flip()

        fall_speed = max(0.1, 0.5 - (game.level - 1) * 0.05)

    pygame.quit()

if __name__ == "__main__":
    main()